from .base import ModuleTestBase


class TestASNBGPView(ModuleTestBase):
    targets = ["8.8.8.8"]
    module_name = "asn"
    config_overrides = {"scope_report_distance": 2}

    response_get_asn_bgpview = {
        "status": "ok",
        "status_message": "Query was successful",
        "data": {
            "ip": "8.8.8.8",
            "ptr_record": "dns.google",
            "prefixes": [
                {
                    "prefix": "8.8.8.0/24",
                    "ip": "8.8.8.0",
                    "cidr": 24,
                    "asn": {"asn": 15169, "name": "GOOGLE", "description": "Google LLC", "country_code": "US"},
                    "name": "LVLT-GOGL-8-8-8",
                    "description": "Google LLC",
                    "country_code": "US",
                }
            ],
            "rir_allocation": {
                "rir_name": "ARIN",
                "country_code": None,
                "ip": "8.0.0.0",
                "cidr": 9,
                "prefix": "8.0.0.0/9",
                "date_allocated": "1992-12-01 00:00:00",
                "allocation_status": "allocated",
            },
            "iana_assignment": {
                "assignment_status": "legacy",
                "description": "Administered by ARIN",
                "whois_server": "whois.arin.net",
                "date_assigned": None,
            },
            "maxmind": {"country_code": None, "city": None},
        },
        "@meta": {"time_zone": "UTC", "api_version": 1, "execution_time": "567.18 ms"},
    }
    response_get_emails_bgpview = {
        "status": "ok",
        "status_message": "Query was successful",
        "data": {
            "asn": 15169,
            "name": "GOOGLE",
            "description_short": "Google LLC",
            "description_full": ["Google LLC"],
            "country_code": "US",
            "website": "https://about.google/intl/en/",
            "email_contacts": ["network-abuse@google.com", "arin-contact@google.com"],
            "abuse_contacts": ["network-abuse@google.com"],
            "looking_glass": None,
            "traffic_estimation": None,
            "traffic_ratio": "Mostly Outbound",
            "owner_address": ["1600 Amphitheatre Parkway", "Mountain View", "CA", "94043", "US"],
            "rir_allocation": {
                "rir_name": "ARIN",
                "country_code": "US",
                "date_allocated": "2000-03-30 00:00:00",
                "allocation_status": "assigned",
            },
            "iana_assignment": {
                "assignment_status": None,
                "description": None,
                "whois_server": None,
                "date_assigned": None,
            },
            "date_updated": "2023-02-07 06:39:11",
        },
        "@meta": {"time_zone": "UTC", "api_version": 1, "execution_time": "56.55 ms"},
    }

    async def setup_after_prep(self, module_test):
        module_test.httpx_mock.add_response(
            url="https://api.bgpview.io/ip/8.8.8.8", json=self.response_get_asn_bgpview
        )
        module_test.httpx_mock.add_response(
            url="https://api.bgpview.io/asn/15169", json=self.response_get_emails_bgpview
        )
        module_test.module.sources = ["bgpview"]

    def check(self, module_test, events):
        assert any(e.type == "ASN" for e in events)
        assert any(e.type == "EMAIL_ADDRESS" for e in events)


class TestASNRipe(ModuleTestBase):
    targets = ["8.8.8.8"]
    module_name = "asn"
    config_overrides = {"scope_report_distance": 2}

    response_get_asn_ripe = {
        "messages": [],
        "see_also": [],
        "version": "1.1",
        "data_call_name": "network-info",
        "data_call_status": "supported",
        "cached": False,
        "data": {"asns": ["15169"], "prefix": "8.8.8.0/24"},
        "query_id": "20230217212133-f278ff23-d940-4634-8115-a64dee06997b",
        "process_time": 5,
        "server_id": "app139",
        "build_version": "live.2023.2.1.142",
        "status": "ok",
        "status_code": 200,
        "time": "2023-02-17T21:21:33.428469",
    }
    response_get_asn_metadata_ripe = {
        "messages": [],
        "see_also": [],
        "version": "4.1",
        "data_call_name": "whois",
        "data_call_status": "supported - connecting to ursa",
        "cached": False,
        "data": {
            "records": [
                [
                    {"key": "ASNumber", "value": "15169", "details_link": None},
                    {"key": "ASName", "value": "GOOGLE", "details_link": None},
                    {"key": "ASHandle", "value": "15169", "details_link": "https://stat.ripe.net/AS15169"},
                    {"key": "RegDate", "value": "2000-03-30", "details_link": None},
                    {
                        "key": "Ref",
                        "value": "https://rdap.arin.net/registry/autnum/15169",
                        "details_link": "https://rdap.arin.net/registry/autnum/15169",
                    },
                    {"key": "source", "value": "ARIN", "details_link": None},
                ],
                [
                    {"key": "OrgAbuseHandle", "value": "ABUSE5250-ARIN", "details_link": None},
                    {"key": "OrgAbuseName", "value": "Abuse", "details_link": None},
                    {"key": "OrgAbusePhone", "value": "+1-650-253-0000", "details_link": None},
                    {
                        "key": "OrgAbuseEmail",
                        "value": "network-abuse@google.com",
                        "details_link": "mailto:network-abuse@google.com",
                    },
                    {
                        "key": "OrgAbuseRef",
                        "value": "https://rdap.arin.net/registry/entity/ABUSE5250-ARIN",
                        "details_link": "https://rdap.arin.net/registry/entity/ABUSE5250-ARIN",
                    },
                    {"key": "source", "value": "ARIN", "details_link": None},
                ],
                [
                    {"key": "OrgName", "value": "Google LLC", "details_link": None},
                    {"key": "OrgId", "value": "GOGL", "details_link": None},
                    {"key": "Address", "value": "1600 Amphitheatre Parkway", "details_link": None},
                    {"key": "City", "value": "Mountain View", "details_link": None},
                    {"key": "StateProv", "value": "CA", "details_link": None},
                    {"key": "PostalCode", "value": "94043", "details_link": None},
                    {"key": "Country", "value": "US", "details_link": None},
                    {"key": "RegDate", "value": "2000-03-30", "details_link": None},
                    {
                        "key": "Comment",
                        "value": "Please note that the recommended way to file abuse complaints are located in the following links.",
                        "details_link": None,
                    },
                    {
                        "key": "Comment",
                        "value": "To report abuse and illegal activity: https://www.google.com/contact/",
                        "details_link": None,
                    },
                    {
                        "key": "Comment",
                        "value": "For legal requests: http://support.google.com/legal",
                        "details_link": None,
                    },
                    {"key": "Comment", "value": "Regards,", "details_link": None},
                    {"key": "Comment", "value": "The Google Team", "details_link": None},
                    {
                        "key": "Ref",
                        "value": "https://rdap.arin.net/registry/entity/GOGL",
                        "details_link": "https://rdap.arin.net/registry/entity/GOGL",
                    },
                    {"key": "source", "value": "ARIN", "details_link": None},
                ],
                [
                    {"key": "OrgTechHandle", "value": "ZG39-ARIN", "details_link": None},
                    {"key": "OrgTechName", "value": "Google LLC", "details_link": None},
                    {"key": "OrgTechPhone", "value": "+1-650-253-0000", "details_link": None},
                    {
                        "key": "OrgTechEmail",
                        "value": "arin-contact@google.com",
                        "details_link": "mailto:arin-contact@google.com",
                    },
                    {
                        "key": "OrgTechRef",
                        "value": "https://rdap.arin.net/registry/entity/ZG39-ARIN",
                        "details_link": "https://rdap.arin.net/registry/entity/ZG39-ARIN",
                    },
                    {"key": "source", "value": "ARIN", "details_link": None},
                ],
                [
                    {"key": "RTechHandle", "value": "ZG39-ARIN", "details_link": None},
                    {"key": "RTechName", "value": "Google LLC", "details_link": None},
                    {"key": "RTechPhone", "value": "+1-650-253-0000", "details_link": None},
                    {"key": "RTechEmail", "value": "arin-contact@google.com", "details_link": None},
                    {
                        "key": "RTechRef",
                        "value": "https://rdap.arin.net/registry/entity/ZG39-ARIN",
                        "details_link": None,
                    },
                    {"key": "source", "value": "ARIN", "details_link": None},
                ],
            ],
            "irr_records": [],
            "authorities": ["arin"],
            "resource": "15169",
            "query_time": "2023-02-17T21:25:00",
        },
        "query_id": "20230217212529-75f57efd-59f4-473f-8bdd-803062e94290",
        "process_time": 268,
        "server_id": "app143",
        "build_version": "live.2023.2.1.142",
        "status": "ok",
        "status_code": 200,
        "time": "2023-02-17T21:25:29.417812",
    }

    async def setup_after_prep(self, module_test):
        module_test.httpx_mock.add_response(
            url="https://stat.ripe.net/data/network-info/data.json?resource=8.8.8.8",
            json=self.response_get_asn_ripe,
        )
        module_test.httpx_mock.add_response(
            url="https://stat.ripe.net/data/whois/data.json?resource=15169",
            json=self.response_get_asn_metadata_ripe,
        )
        module_test.module.sources = ["ripe"]

    def check(self, module_test, events):
        assert any(e.type == "ASN" for e in events)
        assert any(e.type == "EMAIL_ADDRESS" for e in events)



from .base import ModuleTestBase


class TestGitlab(ModuleTestBase):
    targets = ["http://127.0.0.1:8888"]
    modules_overrides = ["gitlab", "httpx"]

    async def setup_before_prep(self, module_test):
        module_test.httpserver.expect_request("/").respond_with_data(headers={"X-Gitlab-Meta": "asdf"})
        module_test.httpserver.expect_request("/api/v4/projects", query_string="simple=true").respond_with_json(
            [
                {
                    "id": 33,
                    "description": None,
                    "name": "bbot",
                    "name_with_namespace": "bbot / BBOT",
                    "path": "bbot",
                    "path_with_namespace": "bbotgroup/bbot",
                    "created_at": "2023-09-07T15:14:05.540Z",
                    "default_branch": "master",
                    "tag_list": [],
                    "topics": [],
                    "ssh_url_to_repo": "git@127.0.0.1:8888:bbot/bbot.git",
                    "http_url_to_repo": "http://127.0.0.1:8888/bbotgroup/bbot.git",
                    "web_url": "http://127.0.0.1:8888/bbotgroup/bbot",
                    "readme_url": "http://127.0.0.1:8888/bbotgroup/bbot/-/blob/master/README.md",
                    "forks_count": 0,
                    "avatar_url": None,
                    "star_count": 1,
                    "last_activity_at": "2024-03-11T19:13:20.691Z",
                    "namespace": {
                        "id": 9,
                        "name": "bbotgroup",
                        "path": "bbotgroup",
                        "kind": "group",
                        "full_path": "bbotgroup",
                        "parent_id": None,
                        "avatar_url": "/uploads/-/system/group/avatar/9/index.png",
                        "web_url": "http://127.0.0.1:8888/groups/bbotgroup",
                    },
                },
            ],
        )
        module_test.httpserver.expect_request("/api/v4/groups", query_string="simple=true").respond_with_json(
            [
                {
                    "id": 9,
                    "web_url": "http://127.0.0.1:8888/groups/bbotgroup",
                    "name": "bbotgroup",
                    "path": "bbotgroup",
                    "description": "OSINT automation for hackers.",
                    "visibility": "public",
                    "share_with_group_lock": False,
                    "require_two_factor_authentication": False,
                    "two_factor_grace_period": 48,
                    "project_creation_level": "developer",
                    "auto_devops_enabled": None,
                    "subgroup_creation_level": "owner",
                    "emails_disabled": False,
                    "emails_enabled": True,
                    "mentions_disabled": None,
                    "lfs_enabled": True,
                    "math_rendering_limits_enabled": True,
                    "lock_math_rendering_limits_enabled": False,
                    "default_branch_protection": 2,
                    "default_branch_protection_defaults": {
                        "allowed_to_push": [{"access_level": 30}],
                        "allow_force_push": True,
                        "allowed_to_merge": [{"access_level": 30}],
                    },
                    "avatar_url": "http://127.0.0.1:8888/uploads/-/system/group/avatar/9/index.png",
                    "request_access_enabled": False,
                    "full_name": "bbotgroup",
                    "full_path": "bbotgroup",
                    "created_at": "2018-05-15T14:31:12.027Z",
                    "parent_id": None,
                    "organization_id": 1,
                    "shared_runners_setting": "enabled",
                    "ldap_cn": None,
                    "ldap_access": None,
                    "marked_for_deletion_on": None,
                    "wiki_access_level": "enabled",
                }
            ]
        )
        module_test.httpserver.expect_request(
            "/api/v4/groups/bbotgroup/projects", query_string="simple=true"
        ).respond_with_json(
            [
                {
                    "id": 33,
                    "description": None,
                    "name": "bbot2",
                    "name_with_namespace": "bbotgroup / bbot2",
                    "path": "bbot2",
                    "path_with_namespace": "bbotgroup/bbot2",
                    "created_at": "2023-09-07T15:14:05.540Z",
                    "default_branch": "master",
                    "tag_list": [],
                    "topics": [],
                    "ssh_url_to_repo": "git@blacklanternsecurity.com:bbotgroup/bbot2.git",
                    "http_url_to_repo": "http://127.0.0.1:8888/bbotgroup/bbot2.git",
                    "web_url": "http://127.0.0.1:8888/bbotgroup/bbot2",
                    "readme_url": "http://127.0.0.1:8888/bbotgroup/bbot2/-/blob/master/README.md",
                    "forks_count": 0,
                    "avatar_url": None,
                    "star_count": 1,
                    "last_activity_at": "2024-03-11T19:13:20.691Z",
                    "namespace": {
                        "id": 9,
                        "name": "bbotgroup",
                        "path": "bbotgroup",
                        "kind": "group",
                        "full_path": "bbotgroup",
                        "parent_id": None,
                        "avatar_url": "/uploads/-/system/group/avatar/9/index.png",
                        "web_url": "http://127.0.0.1:8888/groups/bbotgroup",
                    },
                },
            ]
        )
        module_test.httpserver.expect_request(
            "/api/v4/users/bbotgroup/projects", query_string="simple=true"
        ).respond_with_json(
            [
                {
                    "id": 33,
                    "description": None,
                    "name": "bbot3",
                    "name_with_namespace": "bbotgroup / bbot3",
                    "path": "bbot3",
                    "path_with_namespace": "bbotgroup/bbot3",
                    "created_at": "2023-09-07T15:14:05.540Z",
                    "default_branch": "master",
                    "tag_list": [],
                    "topics": [],
                    "ssh_url_to_repo": "git@blacklanternsecurity.com:bbotgroup/bbot3.git",
                    "http_url_to_repo": "http://127.0.0.1:8888/bbotgroup/bbot3.git",
                    "web_url": "http://127.0.0.1:8888/bbotgroup/bbot3",
                    "readme_url": "http://127.0.0.1:8888/bbotgroup/bbot3/-/blob/master/README.md",
                    "forks_count": 0,
                    "avatar_url": None,
                    "star_count": 1,
                    "last_activity_at": "2024-03-11T19:13:20.691Z",
                    "namespace": {
                        "id": 9,
                        "name": "bbotgroup",
                        "path": "bbotgroup",
                        "kind": "group",
                        "full_path": "bbotgroup",
                        "parent_id": None,
                        "avatar_url": "/uploads/-/system/group/avatar/9/index.png",
                        "web_url": "http://127.0.0.1:8888/groups/bbotgroup",
                    },
                },
            ]
        )

    def check(self, module_test, events):
        assert 1 == len(
            [
                e
                for e in events
                if e.type == "TECHNOLOGY"
                and e.data["technology"] == "GitLab"
                and e.data["url"] == "http://127.0.0.1:8888/"
            ]
        )
        assert 1 == len(
            [
                e
                for e in events
                if e.type == "SOCIAL"
                and e.data["platform"] == "gitlab"
                and e.data["profile_name"] == "bbotgroup"
                and e.data["url"] == "http://127.0.0.1:8888/bbotgroup"
                and str(e.module) == "gitlab"
            ]
        )
        assert 1 == len(
            [
                e
                for e in events
                if e.type == "CODE_REPOSITORY"
                and "git" in e.tags
                and e.data["url"] == "http://127.0.0.1:8888/bbotgroup/bbot"
            ]
        )
        assert 1 == len(
            [
                e
                for e in events
                if e.type == "CODE_REPOSITORY"
                and "git" in e.tags
                and e.data["url"] == "http://127.0.0.1:8888/bbotgroup/bbot2"
            ]
        )
        assert 1 == len(
            [
                e
                for e in events
                if e.type == "CODE_REPOSITORY"
                and "git" in e.tags
                and e.data["url"] == "http://127.0.0.1:8888/bbotgroup/bbot3"
            ]
        )


class TestGitlabDotOrg(ModuleTestBase):
    targets = ["http://127.0.0.1:8888"]
    modules_overrides = ["gitlab", "httpx", "social", "excavate"]

    async def setup_before_prep(self, module_test):
        module_test.httpserver.expect_request("/").respond_with_data("<a href='https://gitlab.org/veilidgroup'/>")
        module_test.httpx_mock.add_response(
            url="https://gitlab.org/api/v4/groups/veilidgroup/projects?simple=true",
            json=[
                {
                    "id": 55490429,
                    "description": None,
                    "name": "Veilid",
                    "name_with_namespace": "Veilid / Veilid",
                    "path": "veilid",
                    "path_with_namespace": "veilidgroup/veilid",
                    "created_at": "2024-03-03T05:22:53.169Z",
                    "default_branch": "master",
                    "tag_list": [],
                    "topics": [],
                    "ssh_url_to_repo": "git@gitlab.org:veilid/veilid.git",
                    "http_url_to_repo": "https://gitlab.org/veilidgroup/veilid.git",
                    "web_url": "https://gitlab.org/veilidgroup/veilid",
                    "readme_url": "https://gitlab.org/veilidgroup/veilid/-/blob/master/README.md",
                    "forks_count": 0,
                    "avatar_url": None,
                    "star_count": 0,
                    "last_activity_at": "2024-03-03T05:22:53.097Z",
                    "namespace": {
                        "id": 66882294,
                        "name": "veilidgroup",
                        "path": "veilidgroup",
                        "kind": "group",
                        "full_path": "veilidgroup",
                        "parent_id": None,
                        "avatar_url": "/uploads/-/system/group/avatar/66882294/signal-2023-07-04-192426_003.jpeg",
                        "web_url": "https://gitlab.org/groups/veilidgroup",
                    },
                },
            ],
        )

    def check(self, module_test, events):
        assert 1 == len(
            [
                e
                for e in events
                if e.type == "SOCIAL"
                and e.data["platform"] == "gitlab"
                and e.data["profile_name"] == "veilidgroup"
                and e.data["url"] == "https://gitlab.org/veilidgroup"
            ]
        )
        assert 1 == len(
            [
                e
                for e in events
                if e.type == "CODE_REPOSITORY"
                and "git" in e.tags
                and e.data["url"] == "https://gitlab.org/veilidgroup/veilid"
                and str(e.module) == "gitlab"
            ]
        )



from bbot.modules.templates.subdomain_enum import subdomain_enum_apikey


class zoomeye(subdomain_enum_apikey):
    watched_events = ["DNS_NAME"]
    produced_events = ["DNS_NAME"]
    flags = ["affiliates", "subdomain-enum", "passive", "safe"]
    meta = {"description": "Query ZoomEye's API for subdomains", "auth_required": True}
    options = {"api_key": "", "max_pages": 20, "include_related": False}
    options_desc = {
        "api_key": "ZoomEye API key",
        "max_pages": "How many pages of results to fetch",
        "include_related": "Include domains which may be related to the target",
    }

    base_url = "https://api.zoomeye.org"

    async def setup(self):
        self.max_pages = self.config.get("max_pages", 20)
        self.headers = {"API-KEY": self.config.get("api_key", "")}
        self.include_related = self.config.get("include_related", False)
        return await super().setup()

    async def ping(self):
        url = f"{self.base_url}/resources-info"
        r = await self.helpers.request(url, headers=self.headers)
        assert int(r.json()["quota_info"]["remain_total_quota"]) > 0, "No quota remaining"

    async def handle_event(self, event):
        query = self.make_query(event)
        results = await self.query(query)
        if results:
            for hostname in results:
                if hostname == event:
                    continue
                tags = []
                if not hostname.endswith(f".{query}"):
                    tags = ["affiliate"]
                await self.emit_event(hostname, "DNS_NAME", event, tags=tags)

    async def query(self, query):
        results = set()
        query_type = 0 if self.include_related else 1
        url = f"{self.base_url}/domain/search?q={self.helpers.quote(query)}&type={query_type}&page=" + "{page}"
        i = 0
        agen = self.helpers.api_page_iter(url, headers=self.headers)
        try:
            async for j in agen:
                r = list(self.parse_results(j))
                if r:
                    results.update(set(r))
                if not r or i >= (self.max_pages - 1):
                    break
                i += 1
        finally:
            agen.aclose()
        return results

    def parse_results(self, r):
        for entry in r.get("list", []):
            yield entry["name"]



import re
from bbot.modules.base import BaseModule


class social(BaseModule):
    watched_events = ["URL_UNVERIFIED"]
    produced_events = ["SOCIAL"]
    meta = {"description": "Look for social media links in webpages"}
    flags = ["passive", "safe", "social-enum"]

    social_media_regex = {
        "linkedin": r"(?:https?://)?(?:www.)?linkedin.com/(?:in|company)/([a-zA-Z0-9-]+)/?",
        "facebook": r"(?:https?://)?(?:www.)?facebook.com/([a-zA-Z0-9.]+)/?",
        "twitter": r"(?:https?://)?(?:www.)?twitter.com/([a-zA-Z0-9_]{1,15})/?",
        "github": r"(?:https?://)?(?:www.)?github.com/([a-zA-Z0-9_-]+)/?",
        "instagram": r"(?:https?://)?(?:www.)?instagram.com/([a-zA-Z0-9_.]+)/?",
        "youtube": r"(?:https?://)?(?:www.)?youtube.com/@([a-zA-Z0-9_]+)/?",
        "bitbucket": r"(?:https?://)?(?:www.)?bitbucket.org/([a-zA-Z0-9_-]+)/?",
        "gitlab": r"(?:https?://)?(?:www.)?gitlab.(?:com|org)/([a-zA-Z0-9_-]+)",
        "discord": r"(?:https?://)?(?:www.)?discord.gg/([a-zA-Z0-9_-]+)",
        "docker": r"(?:https?://)?hub.docker.com/[ru]/([a-zA-Z0-9_-]+)",
        "huggingface": r"(?:https?://)?huggingface.co/([a-zA-Z0-9_-]+)",
    }

    scope_distance_modifier = 1

    async def setup(self):
        self.compiled_regexes = {k: re.compile(v) for k, v in self.social_media_regex.items()}
        return True

    async def handle_event(self, event):
        for platform, regex in self.compiled_regexes.items():
            for match in regex.finditer(event.data):
                url = match.group()
                if not url.startswith("http"):
                    url = f"https://{url}"
                profile_name = match.groups()[0]
                social_event = self.make_event(
                    {"platform": platform, "url": url, "profile_name": profile_name}, "SOCIAL", source=event
                )
                social_event.scope_distance = event.scope_distance
                await self.emit_event(social_event)



import asyncio

from ..bbot_fixtures import *


@pytest.mark.asyncio
async def test_files(bbot_scanner, bbot_config):
    scan1 = bbot_scanner(config=bbot_config)

    # tempfile
    tempfile = scan1.helpers.tempfile(("line1", "line2"), pipe=False)
    assert list(scan1.helpers.read_file(tempfile)) == ["line1", "line2"]
    tempfile = scan1.helpers.tempfile(("line1", "line2"), pipe=True)
    assert list(scan1.helpers.read_file(tempfile)) == ["line1", "line2"]

    # tempfile tail
    results = []
    tempfile = scan1.helpers.tempfile_tail(callback=lambda x: results.append(x))
    with open(tempfile, "w") as f:
        f.write("asdf\n")
    await asyncio.sleep(0.1)
    assert "asdf" in results



from .base import ModuleTestBase


class TestIPStack(ModuleTestBase):
    targets = ["8.8.8.8"]
    config_overrides = {"modules": {"ipstack": {"api_key": "asdf"}}}

    async def setup_before_prep(self, module_test):
        module_test.httpx_mock.add_response(
            url="http://api.ipstack.com/check?access_key=asdf",
            json={
                "ip": "1.2.3.4",
                "type": "ipv4",
                "continent_code": "NA",
                "continent_name": "North America",
                "country_code": "US",
                "country_name": "United States",
                "region_code": "FL",
                "region_name": "Florida",
                "city": "Cape Canaveral",
                "zip": "12345",
                "latitude": 47.89263153076172,
                "longitude": -97.04190063476562,
                "location": {
                    "geoname_id": 5059429,
                    "capital": "Washington D.C.",
                    "languages": [{"code": "en", "name": "English", "native": "English"}],
                    "country_flag": "https://assets.ipstack.com/flags/us.svg",
                    "country_flag_emoji": "\ud83c\uddfa\ud83c\uddf8",
                    "country_flag_emoji_unicode": "U+1F1FA U+1F1F8",
                    "calling_code": "1",
                    "is_eu": False,
                },
            },
        )
        module_test.httpx_mock.add_response(
            url="http://api.ipstack.com/8.8.8.8?access_key=asdf",
            json={
                "ip": "8.8.8.8",
                "type": "ipv4",
                "continent_code": "NA",
                "continent_name": "North America",
                "country_code": "US",
                "country_name": "United States",
                "region_code": "OH",
                "region_name": "Ohio",
                "city": "Glenmont",
                "zip": "44628",
                "latitude": 40.5369987487793,
                "longitude": -82.12859344482422,
                "location": {
                    "geoname_id": None,
                    "capital": "Washington D.C.",
                    "languages": [{"code": "en", "name": "English", "native": "English"}],
                    "country_flag": "https://assets.ipstack.com/flags/us.svg",
                    "country_flag_emoji": "\ud83c\uddfa\ud83c\uddf8",
                    "country_flag_emoji_unicode": "U+1F1FA U+1F1F8",
                    "calling_code": "1",
                    "is_eu": False,
                },
            },
        )

    def check(self, module_test, events):
        assert any(
            e.type == "GEOLOCATION" and e.data["ip"] == "8.8.8.8" and e.data["city"] == "Glenmont" for e in events
        ), "Failed to geolocate IP"



############################################################
#                                                          #
#                                                          #
#    [-] Processing BuiltWith Domains Output               #
#                                                          #
#    [-] 2022.08.19                                        #
#          V05                                             #
#          Black Lantern Security (BLSOPS)                 #
#                                                          #
#                                                          #
############################################################

from bbot.modules.templates.subdomain_enum import subdomain_enum_apikey


class builtwith(subdomain_enum_apikey):
    watched_events = ["DNS_NAME"]
    produced_events = ["DNS_NAME"]
    flags = ["affiliates", "subdomain-enum", "passive", "safe"]
    meta = {"description": "Query Builtwith.com for subdomains", "auth_required": True}
    options = {"api_key": "", "redirects": True}
    options_desc = {"api_key": "Builtwith API key", "redirects": "Also look up inbound and outbound redirects"}
    base_url = "https://api.builtwith.com"

    async def ping(self):
        # builtwith does not have a ping feature, so we skip it to save API credits
        return

    async def handle_event(self, event):
        query = self.make_query(event)
        # domains
        subdomains = await self.query(query, parse_fn=self.parse_domains, request_fn=self.request_domains)
        if subdomains:
            for s in subdomains:
                if s != event:
                    await self.emit_event(s, "DNS_NAME", source=event)
        # redirects
        if self.config.get("redirects", True):
            redirects = await self.query(query, parse_fn=self.parse_redirects, request_fn=self.request_redirects)
            if redirects:
                for r in redirects:
                    if r != event:
                        await self.emit_event(r, "DNS_NAME", source=event, tags=["affiliate"])

    async def request_domains(self, query):
        url = f"{self.base_url}/v20/api.json?KEY={self.api_key}&LOOKUP={query}&NOMETA=yes&NOATTR=yes&HIDETEXT=yes&HIDEDL=yes"
        return await self.request_with_fail_count(url)

    async def request_redirects(self, query):
        url = f"{self.base_url}/redirect1/api.json?KEY={self.api_key}&LOOKUP={query}"
        return await self.request_with_fail_count(url)

    def parse_domains(self, r, query):
        """
        This method returns a set of subdomains.
        Each subdomain is an "FQDN" that was reported in the "Detailed Technology Profile" page on builtwith.com

        Parameters
        ----------
        r (requests Response): The raw requests response from the API
        query (string): The query used against the API
        """
        results_set = set()
        json = r.json()
        if json and isinstance(json, dict):
            results = json.get("Results", [])
            if results:
                for result in results:
                    for chunk in result.get("Result", {}).get("Paths", []):
                        domain = chunk.get("Domain", "")
                        subdomain = chunk.get("SubDomain", "")
                        if domain:
                            if subdomain:
                                domain = f"{subdomain}.{domain}"
                            results_set.add(domain)
            else:
                errors = json.get("Errors", [{}])
                if errors:
                    error = errors[0].get("Message", "Unknown Error")
                    self.verbose(f"No results for {query}: {error}")
        return results_set

    def parse_redirects(self, r, query):
        """
        This method creates a set.
        Each entry in the set is either an Inbound or Outbound Redirect reported in the "Redirect Profile" page on builtwith.com

        Parameters
        ----------
        r (requests Response): The raw requests response from the API
        query (string): The query used against the API

        Returns
        -------
        results (set)
        """
        results = set()
        json = r.json()
        if json and isinstance(json, dict):
            inbound = json.get("Inbound", [])
            outbound = json.get("Outbound", [])
            if inbound:
                for i in inbound:
                    domain = i.get("Domain", "")
                    if domain:
                        results.add(domain)
            if outbound:
                for o in outbound:
                    domain = o.get("Domain", "")
                    if domain:
                        results.add(domain)
        if not results:
            error = json.get("error", "")
            if error:
                self.warning(f"No results for {query}: {error}")
        return results



import json
from pathlib import Path

from bbot.modules.base import BaseModule


class filedownload(BaseModule):
    """
    Watch for common filetypes and download them.

    Capable of identifying interesting files even if the extension is not in the URL.
    E.g. if a PDF is being served at https://evilcorp.com/mypdf, it will still be downloaded and given the proper extension.
    """

    watched_events = ["URL_UNVERIFIED", "HTTP_RESPONSE"]
    produced_events = []
    flags = ["active", "safe", "web-basic", "web-thorough"]
    meta = {"description": "Download common filetypes such as PDF, DOCX, PPTX, etc."}
    options = {
        "extensions": [
            "bak",  #  Backup File
            "bash",  #  Bash Script or Configuration
            "bashrc",  #  Bash Script or Configuration
            "conf",  #  Configuration File
            "cfg",  #  Configuration File
            "crt",  #  Certificate File
            "csv",  #  Comma Separated Values File
            "db",  #  SQLite Database File
            "sqlite",  #  SQLite Database File
            "doc",  #  Microsoft Word Document (Old Format)
            "docx",  #  Microsoft Word Document
            "exe",  #  Windows PE executable
            "ica",  #  Citrix Independent Computing Architecture File
            "indd",  #  Adobe InDesign Document
            "ini",  #  Initialization File
            "jar",  #  Java Archive
            "key",  #  Private Key File
            "pub",  #  Public Key File
            "log",  #  Log File
            "markdown",  #  Markdown File
            "md",  #  Markdown File
            "msi",  # Windows setup file
            "odg",  #  OpenDocument Graphics (LibreOffice, OpenOffice)
            "odp",  #  OpenDocument Presentation (LibreOffice, OpenOffice)
            "ods",  #  OpenDocument Spreadsheet (LibreOffice, OpenOffice)
            "odt",  #  OpenDocument Text (LibreOffice, OpenOffice)
            "pdf",  #  Adobe Portable Document Format
            "pem",  #  Privacy Enhanced Mail (SSL certificate)
            "pps",  #  Microsoft PowerPoint Slideshow (Old Format)
            "ppsx",  #  Microsoft PowerPoint Slideshow
            "ppt",  #  Microsoft PowerPoint Presentation (Old Format)
            "pptx",  #  Microsoft PowerPoint Presentation
            "ps1",  #  PowerShell Script
            "raw",  #  Raw Image File Format
            "rdp",  #  Remote Desktop Protocol File
            "sh",  #  Shell Script
            "sql",  #  SQL Database Dump
            "swp",  #  Swap File (temporary file, often Vim)
            "sxw",  #  OpenOffice.org Writer document
            "tar",  #  Tar Archive
            "tar.gz",  # Gzip-Compressed Tar Archive
            "zip",  #  Zip Archive
            "txt",  #  Plain Text Document
            "vbs",  #  Visual Basic Script
            "wpd",  #  WordPerfect Document
            "xls",  #  Microsoft Excel Spreadsheet (Old Format)
            "xlsx",  #  Microsoft Excel Spreadsheet
            "xml",  #  eXtensible Markup Language File
            "yml",  #  YAML Ain't Markup Language
            "yaml",  #  YAML Ain't Markup Language
        ],
        "max_filesize": "10MB",
    }
    options_desc = {
        "extensions": "File extensions to download",
        "max_filesize": "Cancel download if filesize is greater than this size",
    }

    scope_distance_modifier = 3

    async def setup(self):
        self.extensions = list(set([e.lower().strip(".") for e in self.config.get("extensions", [])]))
        self.max_filesize = self.config.get("max_filesize", "10MB")
        self.download_dir = self.scan.home / "filedownload"
        self.helpers.mkdir(self.download_dir)
        self.urls_downloaded = set()
        self.files_downloaded = 0
        self.mime_db_file = await self.helpers.wordlist(
            "https://raw.githubusercontent.com/jshttp/mime-db/master/db.json"
        )
        self.mime_db = {}
        with open(self.mime_db_file) as f:
            mime_db = json.load(f)
            for content_type, attrs in mime_db.items():
                if "extensions" in attrs and attrs["extensions"]:
                    self.mime_db[content_type] = attrs["extensions"][0].lower()
        return True

    async def filter_event(self, event):
        # accept file download requests from other modules
        if "filedownload" in event.tags:
            return True
        else:
            if event.scope_distance > 1:
                return False, f"{event} not within scope distance"
            elif self.hash_event(event) in self.urls_downloaded:
                return False, f"Already processed {event}"
        return True

    def hash_event(self, event):
        if event.type == "HTTP_RESPONSE":
            return hash(event.data["url"])
        return hash(event.data)

    async def handle_event(self, event):
        if event.type == "URL_UNVERIFIED":
            url_lower = event.data.lower()
            extension_matches = any(url_lower.endswith(f".{e}") for e in self.extensions)
            filedownload_requested = "filedownload" in event.tags
            if extension_matches or filedownload_requested:
                await self.download_file(event.data)
        elif event.type == "HTTP_RESPONSE":
            content_type = event.data["header"].get("content_type", "")
            if content_type:
                url = event.data["url"]
                await self.download_file(url, content_type=content_type)

    async def download_file(self, url, content_type=None):
        orig_filename, file_destination, base_url = self.make_filename(url, content_type=content_type)
        if orig_filename is None:
            return
        result = await self.helpers.download(url, warn=False, filename=file_destination, max_size=self.max_filesize)
        if result:
            self.info(f'Found "{orig_filename}" at "{base_url}", downloaded to {file_destination}')
            self.files_downloaded += 1
        self.urls_downloaded.add(hash(url))

    def make_filename(self, url, content_type=None):
        # first, try to determine original filename
        parsed_url = self.helpers.urlparse(url)
        base_url = f"{parsed_url.scheme}://{parsed_url.netloc}"
        url_path = parsed_url.path.strip("/")
        # try to get extension from URL path
        extension = Path(url_path).suffix.strip(".").lower()
        if extension:
            url_stem = url.rsplit(".", 1)[0]
        else:
            url_stem = str(url)
        filename = f"{self.helpers.make_date()}_{self.helpers.tagify(url_stem)}"
        if not url_path:
            url_path = "unknown"
            filename = f"{filename}-{url_path}"
        # if that fails, try to get it from content type
        if not extension:
            if content_type and content_type in self.mime_db:
                extension = self.mime_db[content_type]

        if (not extension) or (extension not in self.extensions):
            self.debug(f'Extension "{extension}" at url "{url}" not in list of watched extensions.')
            return None, None, None

        orig_filename = Path(url_path).stem
        if extension:
            filename = f"{filename}.{extension}"
            orig_filename = f"{orig_filename}.{extension}"
        return orig_filename, self.download_dir / filename, base_url

    async def report(self):
        if self.files_downloaded > 0:
            self.success(f"Downloaded {self.files_downloaded:,} file(s) to {self.download_dir}")
